// Copyright (c) 2011 Bryan Schumaker.
#include <playlist.h>
#include <library.h>
#include <notify.h>
#include <audio.h>
#include <track.h>
#include <prefs.h>
#include <deck.h>
#include <idle.h>
#include <ban.h>
#include <fs.h>
#include "library.h"

#include <list>
#include <string>
using namespace std;

static unsigned int next_id = 0;
static list<struct libsaria::library::Path> path_list;
libsaria::Playlist lib_playlist(PL_STATIC | PL_NO_DRAIN | PL_SORTED);

libsaria::library::Path *push_path(libsaria::library::Path &path)
{
	libsaria::library::Path *ptr;

	path_list.push_back(path);
	ptr = &path_list.back();
	libsaria::notify(PATH_ADDED, ptr);
	return ptr;
}

struct sort_data {
	list<libsaria::Track *> tracks;
	list<libsaria::Track *> banned;
};

/*
 * TODO: Add tracks directly to a playlist without going through
 *       these intermediate lists.  (Possibly) use insertion sort
 *       for inserting?
 */
static void sort_tracks(libsaria::Track &track, void *data)
{
	struct sort_data *sort = (struct sort_data *)data;
	if (track[TRACK_BANNED])
		sort->banned.push_back(&track);
	else
		sort->tracks.push_back(&track);
}

static void hide_track(libsaria::Track &track, void *data)
{
	if (track[TRACK_BANNED])
		libsaria::ban::get_banned_plist()->remove_track(&track);
	else
		lib_playlist.remove_track(&track);
}

static void delete_track(libsaria::Track &track, void *data)
{
	hide_track(track, NULL);
	libsaria::deck::track_removed(&track);
}

static bool check_trackid(libsaria::Track &track, void *data)
{
	return track[TRACK_ID] == *(unsigned int *)data;
}

void pop_path(libsaria::library::Path *path)
{
	list<libsaria::library::Path>::iterator it;
	for (it = path_list.begin(); it != path_list.end(); it++) {
		if ((*it).id == path->id) {
			libsaria::notify(PATH_DELETED, path);
			path_list.erase(it);
			return;
		}
	}
}

void restore_current()
{
	unsigned int l_id = libsaria::prefs::get("libsaria.current.library");
	unsigned int t_id = libsaria::prefs::get("libsaria.current.track");
	libsaria::Track *track = libsaria::library::lookup(l_id, t_id);
	if (track)
		track->load(false);
}

namespace libsaria
{

	void library::add_path(string dir)
	{
		struct Path path, *path_ptr;

		path.visible = true;
		path.data_state = CLEAN;
		path.id = next_id++;
		path.next_track = 0;
		path.path = dir;

		path_ptr = push_path(path);
		update_path(path_ptr);
	}

	void library::delete_path(Path *path)
	{
		list<Track>::iterator it;
		list<Path>::iterator p_it;

		remove_file(path);
		path->tracks.for_each_item(delete_track, NULL);
		pop_path(path);
	}

	void library::update_path(Path *path)
	{
		if (path)
			do_update_path(path);
	}

	void library::update_all()
	{
		list<Path>::iterator it;
		for (it = path_list.begin(); it != path_list.end(); it++)
			do_update_path(&(*it));
	}

	void library::hide_path(Path *path)
	{
		path->tracks.for_each_item(hide_track, NULL);
		path->visible = false;
		path->data_state = DIRTY;
		save_path(path);
		notify(PATH_UPDATED, path);
	}

	void library::show_path(Path *path)
	{
		struct sort_data sort;

		path->tracks.for_each_item(sort_tracks, &sort);
		lib_playlist.push_back(sort.tracks);
		ban::get_banned_plist()->push_back(sort.banned);

		path->visible = true;
		path->data_state = DIRTY;
		save_path(path);
		notify(PATH_UPDATED, path);
	}

	Track *library::lookup(unsigned int libid, unsigned int trackid)
	{
		list<Path>::iterator p_it;
		ListItem<Track> *track;

		for (p_it = path_list.begin(); p_it != path_list.end(); p_it++) {
			if (p_it->id == libid)
				break;
		}
		if (p_it == path_list.end())
			return NULL;

		track = p_it->tracks.find_item(check_trackid, &trackid);
		if (track == NULL)
			return NULL;
		return &track->get_value();
	}

	void library::set_random(bool random)
	{
		prefs::set("libsaria.library.random", random);
		lib_playlist.set_flag(PL_RANDOM, random);
	}

	Playlist *library::get_playlist()
	{
		return &lib_playlist;
	}

	void library::init()
	{
		IdleCallback *task;
		bool random = prefs::init("libsaria.library.random", true);
		lib_playlist.set_flag(PL_RANDOM, random);

		app::mkdir("library");
		next_id = schedule_load();

		task = new IdleCallback(restore_current);
		task->queue();
	}

}; /* namespace: libsaria */
