// Copyright (c) 2012 Bryan Schumaker.
#include <playlist.h>
#include <library.h>
#include <notify.h>
#include <audio.h>
#include <track.h>
#include <print.h>
#include <deck.h>

#include <list>
#include <sstream>
using namespace std;

static AutoPauseType pause_type;
static unsigned short pause_count = 0;

static list<libsaria::Playlist *> playlist_deck;
static libsaria::Playlist recent_plist(PL_STATIC | PL_NO_DRAIN);

static void renumber_playlists()
{
	list<libsaria::Playlist *>::iterator it;
	int n = 0;

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++)
		(*it)->renumber(n++);
}

static libsaria::Playlist *get_active_playlist()
{
	list<libsaria::Playlist *>::iterator it;

	for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
		if (!(*it)->check_flag(PL_DISABLED))
			return *it;
	}

	return libsaria::library::get_playlist();
}

static bool check_play()
{
	switch(pause_type) {
	case PS_AFTER_N:
		if (pause_count == 0) {
			libsaria::deck::set_pause_type(PS_NONE, 0);
			return false;
		} else {
			pause_count--;
			libsaria::notify(PAUSE_COUNT, &pause_count);
		}
	default:
		break;
	}

	return true;
}

namespace libsaria
{

	void deck::push_front(Playlist *plist)
	{
		playlist_deck.push_front(plist);
	}

	void deck::push_back(Playlist *plist)
	{
		playlist_deck.push_back(plist);
	}

	void deck::garbage_collect()
	{
		int n = 0;
		list<libsaria::Playlist *>::iterator it;

		for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
			if ((*it)->get_size() == 0) {
				(*it)->prepare_for_removal();
				notify(PLAYLIST_DELETE, *it);
				delete (*it);
				it = playlist_deck.erase(it);
				it--;
			} else
				(*it)->renumber(n++);
		}
	}

	void deck::next()
	{
		Playlist *plist = get_active_playlist();
		Track *track = plist->next();

		if (track)
			track->load(check_play());

		garbage_collect();
	}

	void deck::prev()
	{
		Track *track = recent_plist.next();
		if (track)
			track->load_unlisted(true);
	}

	Playlist *deck::new_playlist(list<Track *> &tracks, unsigned int flags, bool front)
	{
		Playlist *plist;

		if (playlist_deck.size() == MAX_PLAYLISTS)
			return NULL;
		plist = new Playlist(flags);

		if (front)
			deck::push_front(plist);
		else
			deck::push_back(plist);
		notify(PLAYLIST_NEW, plist);

		renumber_playlists();

		plist->push_back(tracks);
		return plist;
	}

	void deck::delete_playlist(Playlist *plist)
	{
		plist->prepare_for_removal();
		notify(PLAYLIST_DELETE, plist);
		playlist_deck.remove(plist);
		garbage_collect();
	}

	unsigned int deck::move_playlist(Playlist *plist, unsigned int index)
	{
		list<Playlist *>::iterator it;

		if (index >= playlist_deck.size())
			index = playlist_deck.size() - 1;

		playlist_deck.remove(plist);

		it = playlist_deck.begin();
		for (unsigned int i = 0; i < index; i++)
			it++;

		playlist_deck.insert(it, plist);
		garbage_collect();

		return index;
	}

	unsigned int deck::num_playlists()
	{
		return playlist_deck.size();
	}

	void deck::init()
	{
		app::mkdir("playlist");
	}

	void deck::list_recent(Track *track)
	{
		recent_plist.remove_track(track);
		recent_plist.push_front(track);
		recent_plist.reset_iterator();
	}

	Playlist *deck::get_recent_plist()
	{
		return &recent_plist;
	}

	Playlist *deck::get_playlist(unsigned int n)
	{
		list<libsaria::Playlist *>::iterator it;

		if (n >= playlist_deck.size())
			return NULL;

		it = playlist_deck.begin();
		for (unsigned int i = 0; i < n; i++)
			it++;

		return *it;
	}

	int deck::get_playlist_index(Playlist *plist)
	{
		list<libsaria::Playlist *>::iterator it;
		int n = 0;
		for (it = playlist_deck.begin(); it != playlist_deck.end(); it++) {
			if ((*it) == plist)
				return n;
			n++;
		}
		return -1;
	}

	void deck::track_removed(Track *track)
	{
		list<Playlist *>::iterator it;
		for (it = playlist_deck.begin(); it != playlist_deck.end(); it++)
			(*it)->remove_track(track);
	}

	void deck::set_pause_type(AutoPauseType type, unsigned int count)
	{
		pause_type = type;
		pause_count = count;
		notify(PAUSE_TYPE, &pause_type);
		notify(PAUSE_COUNT, &count);
	}

	AutoPauseType deck::get_pause_type()
	{
		return pause_type;
	}

	unsigned short deck::get_pause_count()
	{
		return pause_count;
	}

}; /* Namespace: libsaria */
